@@ -1,7 +1,10 @@
 # DATA CHECKING ####
+
 # * Error and warning messages for upload ####
 checkDataText_Input <- eventReactive(input$submitButton, {
   error_setting <- F
+  submitCounter$Click <- 1
+  removeCounter$Click <- 0
 
   if (dataset$data_status == "missing"){
     str_errors <- "No data found! Please, upload data. Default dataset GMAT is now in use."
@@ -65,10 +68,56 @@ checkDataText_Input <- eventReactive(input$submitButton, {
     }
   }
 })
+
 output$checkDataText <- renderUI({
   HTML(checkDataText_Input())
 })
 
+# * Click counter ####
+submitCounter <- reactiveValues()
+removeCounter <- reactiveValues()
+submitCounter$Click <- 0
+removeCounter$Click <- 0
+
+# * Render remove button after data load ####
+output$removeBut_output <- renderUI({
+  if(submitCounter$Click > 0) {
+    tagList(actionButton(inputId = "removeButton",
+                         label = "Unload data",
+                         class = "btn btn-large btn-primary",
+                         icon = icon("trash"),
+                         width = "150px"))
+  }
+})
+
+# * Remove loaded data ####
+observeEvent(input$removeButton, {
+  # reset function reset values in input
+  # html function change text in corresponding html tag
+  reset("data")
+  reset("key")
+  reset("groups")
+  reset("criterion_variable")
+  reset("maxOrdinal")
+  reset("minOrdinal")
+  reset("globalMax")
+  reset("globalMin")
+  reset("globalCut")
+  reset("submitButton")
+  html('removedItemsText',html = '')
+  html('checkDataText',html = '')
+  html('checkDataColumns01Text',html = '')
+  html('renderdeleteButtonColumns01',html = '')
+  html('removedItemsText',html = '')
+  html('checkGroupText',html = '')
+  html('renderdeleteButtonGroup',html = '')
+  html('removedGroupText',html = '')
+  submitCounter$Click <- 0
+  removeCounter$Click <- 1
+  removeUI(selector = '#removeButton')
+  removeCounter$Click <- 0
+})
+
 # * Checking uploaded scored data ####
 checkDataColumns01_Input <- reactive({
   data <- binary()
@@ -264,7 +313,9 @@ data_description_Input <- reactive({
                 HCI_ShinyItemAnalysis = "<code>HCI</code> (McFarland et al., 2017) is a real dataset of Homeostasis
                 Concept Inventory (HCI) from <code>ShinyItemAnalysis</code> R package. The dataset represents responses of
                 651 subjects (405 males, 246 females) to multiple-choice test of 20 items. <code>HCI</code> contains
-                criterion variable -  indicator whether student plans to major in the life sciences. ")
+                criterion variable -  indicator whether student plans to major in the life sciences. ",
+				Science_mirt = "<code>Science</code> is a 4-item dataset from library <code>mirt/ltm</code> describing attitude to science and technology. Selected items are <code>Comfort</code>, <code>Work</code>, <code>Future</code>, and <code>Benefit</code>. All items are measured on the same scale with response categories:
+				'strongly disagree', 'disagree to some extent', 'agree to some extent', and 'strongly agree'. See Bartholomew et al. (2002) for more details.")
   txt
 
 })
@@ -289,6 +340,8 @@ output$data_key_file_input <- renderUI({
                        ".tsv"))
 })
 
+
+
 # BASIC SUMMARY ####
 
 # * Main dataset ####
